/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Enjin.h"
#include "Singleton.h"

#define GRAVITY 9.999999

// Detecta se um ponto se encontra em uma regiao retangular

bool inRegion(int x, int y, int regX1, int regY1, int regX2, int regY2) {
    if ((x >= regX1) && (x <= regX2) &&	(y >= regY1) && (y <= regY2)) {
        return true;
    }
    return false;
}

// CLASS Sphere

class Sphere {
private:
    Enjin *enjin;
    Sound *sound1;
    Sound *sound2;
    vector <Sphere*> spheres;
    Image *imagem;

    float velocityX;
    float velocityY;
    float elasticity;
    float attrition;
public:
    int x;
    int y;
    Sphere(vector <Sphere*> s, Image *img, Sound *s1,
           Sound *s2, int posX, int posY, float e);
    void run();
    void draw();
    void kick(float vx, float vy);
};

Sphere::Sphere(vector <Sphere*> s, Image *img, Sound *s1,
               Sound *s2, int posX, int posY, float e) {
    enjin = Singleton<Enjin>::getInstance();
    spheres = s;
    imagem = img;
    sound1 = s1;
    sound2 = s2;
    x = posX;
    y = posY;

    velocityX = 0.0;
    velocityY = 0.0;
    attrition = 0.1;
    elasticity = e;
}

void Sphere::draw() {
    enjin->drawImage(imagem, enjin->screen, x, y, 255);
}

void Sphere::kick(float vx, float vy) {
    velocityX += vx;
    velocityY -= vy;
}

void Sphere::run() {
    // Controla a gravidade

    // a velocity da vertical vai aumentando conforme a gravidade
    // como o Y cresce para baixo, entao fazemos inverso
    velocityY = velocityY - (GRAVITY / 10);
    y = y - velocityY;
    if (y >= (enjin->screen->h - imagem->h)) {
        y = enjin->screen->h - imagem->h;
        velocityY = (velocityY * (-1) - elasticity);

        // se a velocity estiver zerada, entao nao toca o som de quicar
        if (velocityY < 0) velocityY = 0;
        if (velocityY != 0) {
            enjin->playSound(sound1, 0);
        }
    }
    // algo pode influenciar na velocity horizontal
    // mas ela vai diminuindo por causa do attrition e gravidade
    x = x + velocityX;
    if (velocityX > 0) {
        velocityX -= attrition;
    } else if (velocityX < 0) {
        velocityX += attrition;
    }
    if ((x < 0) || (x > enjin->screen->w - imagem->w)) {
        velocityX *= (-1);
    }
    if ((velocityX < 0.1) && (velocityX > -0.1)) {
        velocityX = 0;
    }
    // Verifica se foi clicado com o mouse
    int mX = enjin->getMouseX();
    int mY = enjin->getMouseY();
    int meioX = x + (int) (imagem->w / 2);
    int meioY = y + (int) (imagem->w / 2);
    if (enjin->isMouseDown()) {
        if (inRegion(mX, mY, x, y, meioX, meioY)) {
            kick(20, 20);
            enjin->playSound(sound1, 0);
        } else if (inRegion(mX, mY, meioX, y, x+imagem->w, meioY)) {
            kick(-20, 20);
            enjin->playSound(sound1, 0);
        } else if (inRegion(mX, mY, x, meioY, meioX, y+imagem->h)) {
            kick(20, -20);
            enjin->playSound(sound1, 0);
        } else if (inRegion(mX, mY, meioX, meioY, x+imagem->w, y+imagem->h)) {
            kick(-20, -20);
            enjin->playSound(sound1, 0);
        }
    }
    // Detectar colisoes
    for (uint i = 0; i < spheres.size(); i++) {
        if (spheres[i] != this) {
            int xa1 = x;
            int ya1 = y;
            int xa2 = x + imagem->w;
            int ya2 = y + imagem->h;
            int xb1 = spheres[i]->x;
            int yb1 = spheres[i]->y;
            int xb2 = spheres[i]->x + spheres[i]->imagem->w;
            int yb2 = spheres[i]->y + spheres[i]->imagem->h;

            if ((inRegion(xa1, ya1, xb1, yb1, xb2, yb2)) ||
                    (inRegion(xa1, ya2, xb1, yb1, xb2, yb2)) ||
                    (inRegion(xa2, ya1, xb1, yb1, xb2, yb2)) ||
                    (inRegion(xa2, ya2, xb1, yb1, xb2, yb2))) {
                if (ya1 > yb1) {
                    kick(0, 4);
                    spheres[i]->kick(0, -4);
                } else if (ya1 < yb1) {
                    kick(0, -4);
                    spheres[i]->kick(0, 4);
                }
                if (xa1 > xb1) {
                    kick(4, 0);
                    spheres[i]->kick(-4, 0);
                } else if (xa1 < xb1) {
                    kick(-4, 0);
                    spheres[i]->kick(4, 0);
                }
                enjin->playSound(sound2, 0);
            }
        }
    }
}

// CLASS GRAVITY

class Game {
private:
    Enjin *enjin;
    Image *background;
    Image *sphere;
    Sound *sound1;
    Sound *sound2;
    vector <Sphere*> spheres;
    bool exitGame;
public:
    Game();
    void render();
    void simulateGameWorld();
    void processInput();
    void gameLoop();
    ~Game();
};

Game::Game() {
    enjin = Singleton<Enjin>::getInstance();
    enjin->set("Game", 800, 600);

    sound1 = enjin->loadSound("../../midia/audio/Basso.wav");
    sound2 = enjin->loadSound("../../midia/audio/Glass.wav");

    background = enjin->loadImage("../../midia/img/greenBackground.jpg");
    sphere = enjin->loadImage("../../midia/img/sphere.png");

    spheres.push_back(new Sphere(spheres, sphere, sound1, sound2, 100, 100, 1));
    spheres.push_back(new Sphere(spheres, sphere, sound1, sound2, 350, 350, 1));
    spheres.push_back(new Sphere(spheres, sphere, sound1, sound2, 500, 500, 1));
    spheres.push_back(new Sphere(spheres, sphere, sound1, sound2, 600, 300, 1));
    exitGame = false;
    gameLoop();
}

void Game::render() {
    enjin->drawImage(background, enjin->screen, 0, 0, 255);
    for (uint i = 0; i < spheres.size(); i++) {
        spheres[i]->draw();
    }
    enjin->drawScreen();
}

void Game::simulateGameWorld() {
    if (enjin->isKeyDown(SDLK_ESCAPE)) {
        exitGame = true;
    }
    if (enjin->getMouseButton() == 3) {
        int px = enjin->getMouseX() - (int) (sphere->w / 2);
        int py = enjin->getMouseY() - (int) (sphere->h / 2);
        spheres.push_back(new Sphere(spheres, sphere, sound1, sound2, px, py, 1));
    }
    for (uint i = 0; i < spheres.size(); i++) {
        spheres[i]->run();
    }
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::gameLoop() {
    while (!exitGame) {
        render();
        processInput();
        simulateGameWorld();
    }
}

Game::~Game() {
    spheres.clear();
    //delete enjin;
}

int main (int argc, char* argv[]) {
    Game game;
    return 0;
}
